@@ -26,11 +26,11 @@
 #include <string.h>
 
 #include "e2k-autoconfig.h"
-#include "e2k-utils.h"
+#include <e2k-utils.h>
 #include "exchange-autoconfig-wizard.h"
 
-#include <e-util/e-account.h>
-#include <e-util/e-account-list.h>
+#include <libedataserver/e-account.h>
+#include <libedataserver/e-account-list.h>
 #include <e-util/e-dialog-utils.h>
 #include <libedataserverui/e-passwords.h>
 
